function (loc) {
    loc.__     ( 'MSGID_' );
    loc.__x    ( 'MSGID_x {key1} {key2}', { 'key1' : 'value1', 'key2' : 'value2' } );
    loc.__n    ( 'MSGID_n', 'PLURAL_n', COUNT );
    loc.__nx   ( 'MSGID_nx', 'PLURAL_nx', COUNT );
    loc.__p    ( 'MSGCTXT', 'MSGID_p' );
    loc.__px   ( 'MSGCTXT', 'MSGID_px' );
    loc.__np   ( 'MSGCTXT', 'MSGID_np', 'PLURAL_np', COUNT );
    loc.__npx  ( 'MSGCTXT', 'MSGID_npx', 'PLURAL_npx', COUNT );

    loc.__d    ( 'TEXTDOMAIN', 'MSGID_d' );
    loc.__dx   ( 'TEXTDOMAIN', 'MSGID_dx' );
    loc.__dn   ( 'TEXTDOMAIN', 'MSGID_dn', 'PLURAL_dn', COUNT );
    loc.__dnx  ( 'TEXTDOMAIN', 'MSGID_dnx', 'PLURAL_dnx', COUNT );
    loc.__dp   ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dp' );
    loc.__dpx  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dpx' );
    loc.__dnp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnp', 'PLURAL_dnp', COUNT );
    loc.__dnpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnpx', 'PLURAL_dnpx', COUNT );

    loc.__c    ( 'MSGID_c', 'CATEGORY' );
    loc.__cx   ( 'MSGID_cx', 'CATEGORY' );
    loc.__cn   ( 'MSGID_cn', 'PLURAL_cn', COUNT, 'CATEGORY' );
    loc.__cnx  ( 'MSGID_cnx', 'PLURAL_cnx', COUNT, 'CATEGORY' );
    loc.__cp   ( 'MSGCTXT', 'MSGID_cp', 'CATEGORY' );
    loc.__cpx  ( 'MSGCTXT', 'MSGID_cpx', 'CATEGORY' );
    loc.__cnp  ( 'MSGCTXT', 'MSGID_cnp', 'PLURAL_cnp', COUNT, 'CATEGORY' );
    loc.__cnpx ( 'MSGCTXT', 'MSGID_cnpx', 'PLURAL_cnpx', COUNT, 'CATEGORY' );

    loc.__dc   ( 'TEXTDOMAIN', 'MSGID_dc', 'CATEGORY' );
    loc.__dcx  ( 'TEXTDOMAIN', 'MSGID_dcx', 'CATEGORY' );
    loc.__dcn  ( 'TEXTDOMAIN', 'MSGID_dcn', 'PLURAL_dcn', COUNT, 'CATEGORY' );
    loc.__dcnx ( 'TEXTDOMAIN', 'MSGID_dcnx', 'PLURAL_dcnx', COUNT, 'CATEGORY' );
    loc.__dcp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcp', 'CATEGORY' );
    loc.__dcpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcpx', 'CATEGORY' );
    loc.__dcnp ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnp', 'PLURAL_dcnp', COUNT, 'CATEGORY' );
    loc.__dcnpx( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnpx', 'PLURAL_dcnpx', COUNT, 'CATEGORY' );

    loc.N__     ( 'MSGID_' );
    loc.N__x    ( 'MSGID_x {key1} {key2}', { 'key1' : 'value1', 'key2' : 'value2' } );
    loc.N__n    ( 'MSGID_n', 'PLURAL_n', COUNT );
    loc.N__nx   ( 'MSGID_nx', 'PLURAL_nx', COUNT );
    loc.N__p    ( 'MSGCTXT', 'MSGID_p' );
    loc.N__px   ( 'MSGCTXT', 'MSGID_px' );
    loc.N__np   ( 'MSGCTXT', 'MSGID_np', 'PLURAL_np', COUNT );
    loc.N__npx  ( 'MSGCTXT', 'MSGID_npx', 'PLURAL_npx', COUNT );

    loc.N__d    ( 'TEXTDOMAIN', 'MSGID_d' );
    loc.N__dx   ( 'TEXTDOMAIN', 'MSGID_dx' );
    loc.N__dn   ( 'TEXTDOMAIN', 'MSGID_dn', 'PLURAL_dn', COUNT );
    loc.N__dnx  ( 'TEXTDOMAIN', 'MSGID_dnx', 'PLURAL_dnx', COUNT );
    loc.N__dp   ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dp' );
    loc.N__dpx  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dpx' );
    loc.N__dnp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnp', 'PLURAL_dnp', COUNT );
    loc.N__dnpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnpx', 'PLURAL_dnpx', COUNT );

    loc.N__c    ( 'MSGID_c', 'CATEGORY' );
    loc.N__cx   ( 'MSGID_cx', 'CATEGORY' );
    loc.N__cn   ( 'MSGID_cn', 'PLURAL_cn', COUNT, 'CATEGORY' );
    loc.N__cnx  ( 'MSGID_cnx', 'PLURAL_cnx', COUNT, 'CATEGORY' );
    loc.N__cp   ( 'MSGCTXT', 'MSGID_cp', 'CATEGORY' );
    loc.N__cpx  ( 'MSGCTXT', 'MSGID_cpx', 'CATEGORY' );
    loc.N__cnp  ( 'MSGCTXT', 'MSGID_cnp', 'PLURAL_cnp', COUNT, 'CATEGORY' );
    loc.N__cnpx ( 'MSGCTXT', 'MSGID_cnpx', 'PLURAL_cnpx', COUNT, 'CATEGORY' );

    loc.N__dc   ( 'TEXTDOMAIN', 'MSGID_dc', 'CATEGORY' );
    loc.N__dcx  ( 'TEXTDOMAIN', 'MSGID_dcx', 'CATEGORY' );
    loc.N__dcn  ( 'TEXTDOMAIN', 'MSGID_dcn', 'PLURAL_dcn', COUNT, 'CATEGORY' );
    loc.N__dcnx ( 'TEXTDOMAIN', 'MSGID_dcnx', 'PLURAL_dcnx', COUNT, 'CATEGORY' );
    loc.N__dcp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcp', 'CATEGORY' );
    loc.N__dcpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcpx', 'CATEGORY' );
    loc.N__dcnp ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnp', 'PLURAL_dcnp', COUNT, 'CATEGORY' );
    loc.N__dcnpx( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnpx', 'PLURAL_dcnpx', COUNT, 'CATEGORY' );
}
