function (loc) {
    loc.loc_     ( 'MSGID_' );
    loc.loc_x    ( 'MSGID_x {key1} {key2}', { 'key1' : 'value1', 'key2' : 'value2' } );
    loc.loc_n    ( 'MSGID_n', 'PLURAL_n', COUNT );
    loc.loc_nx   ( 'MSGID_nx', 'PLURAL_nx', COUNT );
    loc.loc_p    ( 'MSGCTXT', 'MSGID_p' );
    loc.loc_px   ( 'MSGCTXT', 'MSGID_px' );
    loc.loc_np   ( 'MSGCTXT', 'MSGID_np', 'PLURAL_np', COUNT );
    loc.loc_npx  ( 'MSGCTXT', 'MSGID_npx', 'PLURAL_npx', COUNT );

    loc.loc_d    ( 'TEXTDOMAIN', 'MSGID_d' );
    loc.loc_dx   ( 'TEXTDOMAIN', 'MSGID_dx' );
    loc.loc_dn   ( 'TEXTDOMAIN', 'MSGID_dn', 'PLURAL_dn', COUNT );
    loc.loc_dnx  ( 'TEXTDOMAIN', 'MSGID_dnx', 'PLURAL_dnx', COUNT );
    loc.loc_dp   ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dp' );
    loc.loc_dpx  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dpx' );
    loc.loc_dnp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnp', 'PLURAL_dnp', COUNT );
    loc.loc_dnpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnpx', 'PLURAL_dnpx', COUNT );

    loc.loc_c    ( 'MSGID_c', 'CATEGORY' );
    loc.loc_cx   ( 'MSGID_cx', 'CATEGORY' );
    loc.loc_cn   ( 'MSGID_cn', 'PLURAL_cn', COUNT, 'CATEGORY' );
    loc.loc_cnx  ( 'MSGID_cnx', 'PLURAL_cnx', COUNT, 'CATEGORY' );
    loc.loc_cp   ( 'MSGCTXT', 'MSGID_cp', 'CATEGORY' );
    loc.loc_cpx  ( 'MSGCTXT', 'MSGID_cpx', 'CATEGORY' );
    loc.loc_cnp  ( 'MSGCTXT', 'MSGID_cnp', 'PLURAL_cnp', COUNT, 'CATEGORY' );
    loc.loc_cnpx ( 'MSGCTXT', 'MSGID_cnpx', 'PLURAL_cnpx', COUNT, 'CATEGORY' );

    loc.loc_dc   ( 'TEXTDOMAIN', 'MSGID_dc', 'CATEGORY' );
    loc.loc_dcx  ( 'TEXTDOMAIN', 'MSGID_dcx', 'CATEGORY' );
    loc.loc_dcn  ( 'TEXTDOMAIN', 'MSGID_dcn', 'PLURAL_dcn', COUNT, 'CATEGORY' );
    loc.loc_dcnx ( 'TEXTDOMAIN', 'MSGID_dcnx', 'PLURAL_dcnx', COUNT, 'CATEGORY' );
    loc.loc_dcp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcp', 'CATEGORY' );
    loc.loc_dcpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcpx', 'CATEGORY' );
    loc.loc_dcnp ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnp', 'PLURAL_dcnp', COUNT, 'CATEGORY' );
    loc.loc_dcnpx( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnpx', 'PLURAL_dcnpx', COUNT, 'CATEGORY' );

    loc.Nloc_     ( 'MSGID_' );
    loc.Nloc_x    ( 'MSGID_x {key1} {key2}', { 'key1' : 'value1', 'key2' : 'value2' } );
    loc.Nloc_n    ( 'MSGID_n', 'PLURAL_n', COUNT );
    loc.Nloc_nx   ( 'MSGID_nx', 'PLURAL_nx', COUNT );
    loc.Nloc_p    ( 'MSGCTXT', 'MSGID_p' );
    loc.Nloc_px   ( 'MSGCTXT', 'MSGID_px' );
    loc.Nloc_np   ( 'MSGCTXT', 'MSGID_np', 'PLURAL_np', COUNT );
    loc.Nloc_npx  ( 'MSGCTXT', 'MSGID_npx', 'PLURAL_npx', COUNT );

    loc.Nloc_d    ( 'TEXTDOMAIN', 'MSGID_d' );
    loc.Nloc_dx   ( 'TEXTDOMAIN', 'MSGID_dx' );
    loc.Nloc_dn   ( 'TEXTDOMAIN', 'MSGID_dn', 'PLURAL_dn', COUNT );
    loc.Nloc_dnx  ( 'TEXTDOMAIN', 'MSGID_dnx', 'PLURAL_dnx', COUNT );
    loc.Nloc_dp   ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dp' );
    loc.Nloc_dpx  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dpx' );
    loc.Nloc_dnp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnp', 'PLURAL_dnp', COUNT );
    loc.Nloc_dnpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dnpx', 'PLURAL_dnpx', COUNT );

    loc.Nloc_c    ( 'MSGID_c', 'CATEGORY' );
    loc.Nloc_cx   ( 'MSGID_cx', 'CATEGORY' );
    loc.Nloc_cn   ( 'MSGID_cn', 'PLURAL_cn', COUNT, 'CATEGORY' );
    loc.Nloc_cnx  ( 'MSGID_cnx', 'PLURAL_cnx', COUNT, 'CATEGORY' );
    loc.Nloc_cp   ( 'MSGCTXT', 'MSGID_cp', 'CATEGORY' );
    loc.Nloc_cpx  ( 'MSGCTXT', 'MSGID_cpx', 'CATEGORY' );
    loc.Nloc_cnp  ( 'MSGCTXT', 'MSGID_cnp', 'PLURAL_cnp', COUNT, 'CATEGORY' );
    loc.Nloc_cnpx ( 'MSGCTXT', 'MSGID_cnpx', 'PLURAL_cnpx', COUNT, 'CATEGORY' );

    loc.Nloc_dc   ( 'TEXTDOMAIN', 'MSGID_dc', 'CATEGORY' );
    loc.Nloc_dcx  ( 'TEXTDOMAIN', 'MSGID_dcx', 'CATEGORY' );
    loc.Nloc_dcn  ( 'TEXTDOMAIN', 'MSGID_dcn', 'PLURAL_dcn', COUNT, 'CATEGORY' );
    loc.Nloc_dcnx ( 'TEXTDOMAIN', 'MSGID_dcnx', 'PLURAL_dcnx', COUNT, 'CATEGORY' );
    loc.Nloc_dcp  ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcp', 'CATEGORY' );
    loc.Nloc_dcpx ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcpx', 'CATEGORY' );
    loc.Nloc_dcnp ( 'TEXTDOMAIN', 'MSGCTXT', 'MSGID_dcnp', 'PLURAL_dcnp', COUNT, 'CATEGORY' );
    loc.Nloc_dcnpx( 
        // comment
        'TEXTDOMAIN',
        // comment
        'MSGCTXT',
        /*
        comment
        */
        'MSGID_dcnpx',
        // comment
        'PLURAL_dcnpx',
        // comment
        COUNT,
        // comment
        'CATEGORY'
    );
}
